import 'package:equatable/equatable.dart';
import 'package:syncrow_web/pages/device_managment/ac/model/ac_model.dart';

abstract class AcsState extends Equatable {
  const AcsState();

  @override
  List<Object> get props => [];
}

class AcsInitialState extends AcsState {}

class AcsLoadingState extends AcsState {}

class ACStatusLoaded extends AcsState {
  final AcStatusModel status;
  final DateTime timestamp;

  ACStatusLoaded(this.status) : timestamp = DateTime.now();

  @override
  List<Object> get props => [status, timestamp];
}

class AcsFailedState extends AcsState {
  final String error;

  const AcsFailedState({required this.error});

  @override
  List<Object> get props => [error];
}
